@@ -17,7 +17,8 @@ typedef enum {
 	V_JSON,
 	V_BIRD,
 	V_OPENBGPD,
-	V_FORMAT
+	V_FORMAT,
+	V_NOKIA
 } bgpq_vendor_t;
 
 typedef enum {
