@@ -323,6 +323,91 @@ bgpq3_print_openbgpd_oaspath(FILE* f, struct bgpq_expander* b)
 	return 0;
 };
 
+int
+bgpq3_print_nokia_aspath(FILE* f, struct bgpq_expander* b)
+{
+	int nc=0, lineNo=1, i, j, k;
+
+	fprintf(f,"configure router policy-options\nbegin\nno as-path-group \"%s\"\n",
+		b->name ? b->name : "NN");
+	fprintf(f,"as-path-group \"%s\"\n", b->name ? b->name : "NN");
+
+	if(b->asn32s[b->asnumber/65536] &&
+		b->asn32s[b->asnumber/65535][(b->asnumber%65536)/8]&
+		(0x80>>(b->asnumber%8))) {
+		fprintf(f,"  entry %u expression \"%u+\"\n", lineNo, b->asnumber);
+		lineNo++;
+	};
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					if(k*65536+i*8+j==b->asnumber) continue;
+					if(!nc) {
+						fprintf(f,"  entry %u expression \"%u.*[%u",
+							lineNo,b->asnumber,k*65536+i*8+j);
+					} else {
+						fprintf(f," %u",k*65536+i*8+j);
+					};
+					nc++;
+					if(nc==b->aswidth) {
+						fprintf(f,"]\"\n");
+						nc=0;
+						lineNo++;
+					};
+				};
+			};
+		};
+	};
+	if(nc) fprintf(f,"]\"\n");
+	fprintf(f,"exit\ncommit\n");
+	return 0;
+};
+
+int
+bgpq3_print_nokia_oaspath(FILE* f, struct bgpq_expander* b)
+{
+	int nc=0, lineNo=1, i, j, k;
+
+	fprintf(f,"configure router policy-options\nbegin\nno as-path-group \"%s\"\n",
+		b->name ? b->name : "NN");
+	fprintf(f,"as-path-group \"%s\"\n", b->name ? b->name : "NN");
+
+	if(b->asn32s[b->asnumber/65536] &&
+		b->asn32s[b->asnumber/65536][(b->asnumber%65536)/8]&
+		(0x80>>(b->asnumber%8))) {
+		fprintf(f,"  entry %u expression \"%u+\"\n", lineNo, b->asnumber);
+		lineNo++;
+	};
+	for(k=0;k<65536;k++) {
+		if(!b->asn32s[k]) continue;
+
+		for(i=0;i<8192;i++) {
+			for(j=0;j<8;j++) {
+				if(b->asn32s[k][i]&(0x80>>j)) {
+					if(k*65536+i*8+j==b->asnumber) continue;
+					if(!nc) {
+						fprintf(f,"  entry %u expression \".*[%u",
+							lineNo,k*65536+i*8+j);
+					} else {
+						fprintf(f," %u",k*65536+i*8+j);
+					}
+					nc++;
+					if(nc==b->aswidth) {
+						fprintf(f,"]\"\n");
+						nc=0;
+						lineNo++;
+					};
+				};
+			};
+		};
+	};
+	if(nc) fprintf(f,"]\"\n");
+	fprintf(f,"exit\ncommit\n");
+	return 0;
+};
+
 int
 bgpq3_print_aspath(FILE* f, struct bgpq_expander* b)
 {
@@ -338,6 +423,8 @@ bgpq3_print_aspath(FILE* f, struct bgpq_expander* b)
 		return bgpq3_print_bird_aspath(f,b);
 	} else if(b->vendor==V_OPENBGPD) {
 		return bgpq3_print_openbgpd_aspath(f,b);
+	} else if(b->vendor==V_NOKIA) {
+		return bgpq3_print_nokia_aspath(f,b);
 	} else {
 		sx_report(SX_FATAL,"Unknown vendor %i\n", b->vendor);
 	};
@@ -355,6 +442,8 @@ bgpq3_print_oaspath(FILE* f, struct bgpq_expander* b)
 		return bgpq3_print_cisco_xr_oaspath(f,b);
 	} else if(b->vendor==V_OPENBGPD) {
 		return bgpq3_print_openbgpd_oaspath(f,b);
+	} else if(b->vendor==V_NOKIA) {
+		return bgpq3_print_nokia_oaspath(f,b);
 	} else {
 		sx_report(SX_FATAL,"Unknown vendor %i\n", b->vendor);
 	};
@@ -689,6 +778,45 @@ checkSon:
 		bgpq3_print_ceacl(n->son,ff);
 };
 
+void
+bgpq3_print_nokia_ipfilter(struct sx_radix_node* n, void* ff)
+{
+	char prefix[128];
+	FILE* f=(FILE*)ff;
+	if(n->isGlue) goto checkSon;
+	if(!f) f=stdout;
+	sx_prefix_snprintf(&n->prefix,prefix,sizeof(prefix));
+	fprintf(f,"    prefix %s\n", prefix);
+checkSon:
+	if(n->son)
+		bgpq3_print_nokia_ipfilter(n->son, ff);
+};
+
+void
+bgpq3_print_nokia_prefix(struct sx_radix_node* n, void* ff)
+{
+	char prefix[128];
+	FILE* f=(FILE*)ff;
+	if(n->isGlue) goto checkSon;
+	if(!f) f=stdout;
+	sx_prefix_snprintf(&n->prefix,prefix,sizeof(prefix));
+	if(!n->isAggregate) {
+		fprintf(f,"    prefix %s exact\n", prefix);
+	} else {
+		if(n->aggregateLow>n->prefix.masklen) {
+			fprintf(f,"    prefix %s prefix-length-range %u-%u\n",
+				prefix,n->aggregateLow,n->aggregateHi);
+		} else {
+			fprintf(f,"    prefix %s prefix-length-range %u-%u\n",
+				prefix, n->prefix.masklen, n->aggregateHi);
+		};
+	};
+checkSon:
+	if(n->son)
+		bgpq3_print_nokia_prefix(n->son, ff);
+
+};
+
 int
 bgpq3_print_juniper_prefixlist(FILE* f, struct bgpq_expander* b)
 {
@@ -840,6 +968,18 @@ bgpq3_print_format_prefixlist(FILE* f, struct bgpq_expander* b)
 	return 0;
 };
 
+int
+bgpq3_print_nokia_prefixlist(FILE* f, struct bgpq_expander* b)
+{
+	bname=b->name ? b->name : "NN";
+	fprintf(f,"configure router policy-options\nbegin\nno prefix-list \"%s\"\n",
+		bname);
+	fprintf(f,"prefix-list \"%s\"\n", bname);
+	sx_radix_tree_foreach(b->tree,bgpq3_print_nokia_prefix,f);
+	fprintf(f,"exit\ncommit\n");
+	return 0;
+};
+
 int
 bgpq3_print_cisco_eacl(FILE* f, struct bgpq_expander* b)
 {
@@ -855,6 +995,22 @@ bgpq3_print_cisco_eacl(FILE* f, struct bgpq_expander* b)
 	return 0;
 };
 
+int
+bgpq3_print_nokia_ipprefixlist(FILE* f, struct bgpq_expander* b)
+{
+	bname=b->name ? b->name : "NN";
+	fprintf(f,"configure filter match-list\nno %s-prefix-list \"%s\"\n",
+		 b->tree->family==AF_INET?"ip":"ipv6", bname);
+	fprintf(f,"%s-prefix-list \"%s\" create\n", b->tree->family==AF_INET?"ip":"ipv6", bname);
+	if (!sx_radix_tree_empty(b->tree)) {
+		sx_radix_tree_foreach(b->tree,bgpq3_print_nokia_ipfilter,f);
+	} else {
+		fprintf(f,"# generated ip-prefix-list %s is empty\n", bname);
+	};
+	fprintf(f,"exit\n");
+	return 0;
+};
+
 int
 bgpq3_print_prefixlist(FILE* f, struct bgpq_expander* b)
 {
@@ -866,6 +1022,7 @@ bgpq3_print_prefixlist(FILE* f, struct bgpq_expander* b)
 		case V_BIRD: return bgpq3_print_bird_prefixlist(f,b);
 		case V_OPENBGPD: return bgpq3_print_openbgpd_prefixlist(f,b);
 		case V_FORMAT: return bgpq3_print_format_prefixlist(f,b);
+		case V_NOKIA: return bgpq3_print_nokia_prefixlist(f,b);
 	};
 	return 0;
 };
@@ -881,6 +1038,7 @@ bgpq3_print_eacl(FILE* f, struct bgpq_expander* b)
 		case V_BIRD: sx_report(SX_FATAL, "unreachable point\n");
 		case V_OPENBGPD: return bgpq3_print_openbgpd_prefixlist(f,b);
 		case V_FORMAT: sx_report(SX_FATAL, "unreachable point\n");
+		case V_NOKIA: return bgpq3_print_nokia_ipprefixlist(f,b);
 	};
 	return 0;
 };
